using System;
using System.Collections;
using Server.Targeting;
using Server.Network;
using Server.Misc;
using Server.Items;


namespace Server.Spells.Sixth
{
	public class SleepFieldSpell : Spell
	{
		
		private static SpellInfo m_Info = new SpellInfo(
		                                                "Sleep Field", "In Zu Grav",
		                                                SpellCircle.Sixth,
		                                                230,
		                                                9052,
		                                                false,
		                                                Reagent.BlackPearl,
		                                                Reagent.Ginseng,
		                                                Reagent.SpidersSilk
		                                               );
		
		public SleepFieldSpell( Mobile caster, Item scroll ) : base( caster, scroll, m_Info )
		{
		}
		
		public override void OnCast()
		{
			if ( Server.Spells.AllSpells.CheckRestriction( Caster, 99 ) == false )
				return;
			
			Caster.Target = new InternalTarget( this );
		}
		
		public void Target( IPoint3D p )
		{
			if ( !Caster.CanSee( p ) )
			{
				Caster.SendLocalizedMessage( 500237 ); // Target can not be seen.
			}
			else if ( SpellHelper.CheckTown( p, Caster ) && CheckSequence() )
			{
				
				if(this.Scroll!=null)
					Scroll.Consume();
				SpellHelper.Turn( Caster, p );
				
				SpellHelper.GetSurfaceTop( ref p );
				
				int dx = Caster.Location.X - p.X;
				int dy = Caster.Location.Y - p.Y;
				int rx = (dx - dy) * 44;
				int ry = (dx + dy) * 44;
				
				bool eastToWest;
				
				if ( rx >= 0 && ry >= 0 )
				{
					eastToWest = false;
				}
				else if ( rx >= 0 )
				{
					eastToWest = true;
				}
				else if ( ry >= 0 )
				{
					eastToWest = true;
				}
				else
				{
					eastToWest = false;
				}
				
				Effects.PlaySound( p, Caster.Map, 0x20B );
				
				int itemID = eastToWest ? 0x3915 : 0x3922;
				
				TimeSpan duration = TimeSpan.FromSeconds( 3.0 + (Caster.Skills[SkillName.Magery].Value * 0.4) );
				
				for ( int i = -2; i <= 2; ++i )
				{
					Point3D loc = new Point3D( eastToWest ? p.X + i : p.X, eastToWest ? p.Y : p.Y + i, p.Z );
					
					new InternalItem( itemID, loc, Caster, Caster.Map, duration, i );
				}
			}
			
			FinishSequence();
		}
		
		[DispellableField]
		private class InternalItem : Item
		{
			private Timer m_Timer;
			private DateTime m_End;
			private Mobile m_Caster;
			private SleepingBody m_Body;
			
			public override bool BlocksFit{ get{ return true; } }
			
			public InternalItem( int itemID, Point3D loc, Mobile caster, Map map, TimeSpan duration, int val ) : base( itemID )
			{
				bool canFit = SpellHelper.AdjustField( ref loc, map, 12, false );
				
				Visible = false;
				Movable = false;
				Light = LightType.Circle300;
				Hue=1169;
				Name="sleep field";
				
				MoveToWorld( loc, map );
				
				m_Caster = caster;
				
				m_End = DateTime.Now + duration;
				
				m_Timer = new InternalTimer( this, TimeSpan.FromSeconds( Math.Abs( val ) * 0.2 ), caster.InLOS( this ), canFit );
				m_Timer.Start();
			}
			
			public override void OnAfterDelete()
			{
				base.OnAfterDelete();
				
				if ( m_Timer != null )
					m_Timer.Stop();
			}
			
			public InternalItem( Serial serial ) : base( serial )
			{
			}
			
			public override void Serialize( GenericWriter writer )
			{
				base.Serialize( writer );
				
				writer.Write( (int) 1 ); // version
				
				writer.Write( m_Caster );
				writer.WriteDeltaTime( m_End );
			}
			
			public override void Deserialize( GenericReader reader )
			{
				base.Deserialize( reader );
				
				int version = reader.ReadInt();
				
				switch ( version )
				{
					case 1:
						{
							m_Caster = reader.ReadMobile();
							
							goto case 0;
						}
					case 0:
						{
							m_End = reader.ReadDeltaTime();
							
							m_Timer = new InternalTimer( this, TimeSpan.Zero, true, true );
							m_Timer.Start();
							
							break;
						}
				}
			}
			
			public void ApplySleepTo( Mobile m )
			{
				m.Hidden = true;
				m.Frozen=true;
				m.Squelched=true;
				
				SleepingBody body = new SleepingBody(m, m.Blessed);
				
				body.Map=m.Map;
				body.Location=m.Location;
				m_Body=body;
				m.Z-=100;
				
				
				m.SendMessage("You fall asleep");
				
				RemoveTimer( m );
				
				TimeSpan duration = TimeSpan.FromSeconds( m_Caster.Skills[SkillName.Magery].Value * 1.2 ); // 120% of magery
				
				Timer t = new BodyTimer( m, duration, m_Body );
				
				m_Table[m] = t;
				
				t.Start();
			}
			private static Hashtable m_Table = new Hashtable();
			
			public static void RemoveTimer( Mobile m )
			{
				Timer t = (Timer)m_Table[m];
				
				if ( t != null )
				{
					t.Stop();
					m_Table.Remove( m );
				}
			}
			
			private class BodyTimer : Timer
			{
				private Mobile m_Mobile;
				private SleepingBody m_Body;
				
				public BodyTimer( Mobile m, TimeSpan duration, SleepingBody body ) : base( duration )
				{
					m_Mobile = m;
					m_Body=body;
				}
				
				protected override void OnTick()
				{
					m_Mobile.RevealingAction();
					m_Mobile.Frozen=false;
					m_Mobile.Squelched=false;
					m_Mobile.Z=m_Body.Z;
					
					if(m_Body!=null)
					{
						m_Body.Delete();
						m_Mobile.SendMessage("You wake up!");
					}
					RemoveTimer( m_Mobile );
				}
			}
			public override bool OnMoveOver( Mobile m )
			{
				if ( Visible && m_Caster != null && SpellHelper.ValidIndirectTarget( m_Caster, m ) && m_Caster.CanBeHarmful( m, false ) )
				{
					
					ApplySleepTo( m );
					m.PlaySound( 0x474 );
				}
				m.Hidden=true;
				return false;
			}
			
			private class InternalTimer : Timer
			{
				private InternalItem m_Item;
				private bool m_InLOS, m_CanFit;
				
				private static Queue m_Queue = new Queue();
				
				public InternalTimer( InternalItem item, TimeSpan delay, bool inLOS, bool canFit ) : base( delay, TimeSpan.FromSeconds( 1.5 ) )
				{
					m_Item = item;
					m_InLOS = inLOS;
					m_CanFit = canFit;
					
					Priority = TimerPriority.FiftyMS;
				}
				
				protected override void OnTick()
				{
					if ( m_Item.Deleted )
						return;
					
					if ( !m_Item.Visible )
					{
						if ( m_InLOS && m_CanFit )
							m_Item.Visible = true;
						else
							m_Item.Delete();
						
						if ( !m_Item.Deleted )
						{
							m_Item.ProcessDelta();
							Effects.SendLocationParticles( EffectItem.Create( m_Item.Location, m_Item.Map, EffectItem.DefaultDuration ), 0x376A, 9, 10, 5040 );
						}
					}
					else if ( DateTime.Now > m_Item.m_End )
					{
						m_Item.Delete();
						Stop();
					}
					else
					{
						Map map = m_Item.Map;
						Mobile caster = m_Item.m_Caster;
						
						if ( map != null && caster != null )
						{
							bool eastToWest = ( m_Item.ItemID == 0x3915 );
							IPooledEnumerable eable = map.GetMobilesInBounds( new Rectangle2D( m_Item.X - (eastToWest ? 0 : 1), m_Item.Y - (eastToWest ? 1 : 0), (eastToWest ? 1 : 2), (eastToWest ? 2 : 1) ) );;
							
							foreach ( Mobile m in eable )
							{
								if ( (m.Z + 16) > m_Item.Z && (m_Item.Z + 12) > m.Z && SpellHelper.ValidIndirectTarget( caster, m ) && caster.CanBeHarmful( m, false ) )
									m_Queue.Enqueue( m );
							}
							
							eable.Free();
							
							while ( m_Queue.Count > 0 )
							{
								Mobile m = (Mobile)m_Queue.Dequeue();
								
								m.PlaySound( 0x3C4 );
								m_Item.ApplySleepTo( m );
								
							}
						}
					}
				}
			}
		}
		
		private class InternalTarget : Target
		{
			private SleepFieldSpell m_Owner;
			
			public InternalTarget( SleepFieldSpell owner ) : base( 12, true, TargetFlags.None )
			{
				m_Owner = owner;
			}
			
			protected override void OnTarget( Mobile from, object o )
			{
				if ( o is IPoint3D )
					m_Owner.Target( (IPoint3D)o );
			}
			
			protected override void OnTargetFinish( Mobile from )
			{
				m_Owner.FinishSequence();
			}
		}
	}
}
